from qtpy import QtCore, QtWidgets, QtGui

from .g27q import G27Q
from .ui_monctl import Ui_Form

class MyWindow(QtWidgets.QWidget, Ui_Form):
    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self.bslider.valueChanged.connect(self.bvalue.setNum)
        self.cslider.valueChanged.connect(self.cvalue.setNum)
        self.sslider.valueChanged.connect(self.svalue.setNum)
        self.blslider.valueChanged.connect(self.blvalue.setNum)

        self.initValues()
        self.connectChanges()

    def initValues(self):
        with G27Q() as monitor:
            self.bslider.setValue(monitor.get_property("Brightness"))
            self.cslider.setValue(monitor.get_property("Contrast"))
            self.sslider.setValue(monitor.get_property("Sharpness"))
            self.blslider.setValue(monitor.get_property("BlueLight"))

    def connectChanges(self):
        self.bslider.valueChanged.connect(self.set_brightness)
        self.cslider.valueChanged.connect(self.set_contrast)
        self.sslider.valueChanged.connect(self.set_sharpness)
        self.blslider.valueChanged.connect(self.set_blueLight)

    def set_monitorProperty(self, property: str, value: int):
        with G27Q() as monitor:
            monitor.set_property(property,value)

    def set_brightness(self,value):
        self.set_monitorProperty("Brightness", value)
    
    def set_contrast(self,value):
        self.set_monitorProperty("Contrast", value)
    
    def set_sharpness(self,value):
        self.set_monitorProperty("Sharpness", value)
    
    def set_blueLight(self,value):
        self.set_monitorProperty("BlueLight", value)


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'monctlHRvwHx.ui'
##
## Created by: Qt User Interface Compiler version 6.4.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from qtpy.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from qtpy.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from qtpy.QtWidgets import (QApplication, QGridLayout, QLabel, QSizePolicy,
    QSlider, QToolButton, QVBoxLayout, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(318, 331)
        self.verticalLayout = QVBoxLayout(Form)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.brightness = QWidget(Form)
        self.brightness.setObjectName(u"brightness")
        self.gridLayout = QGridLayout(self.brightness)
        self.gridLayout.setObjectName(u"gridLayout")
        self.bslider = QSlider(self.brightness)
        self.bslider.setObjectName(u"bslider")
        self.bslider.setMaximum(100)
        self.bslider.setOrientation(Qt.Horizontal)

        self.gridLayout.addWidget(self.bslider, 2, 0, 1, 2)

        self.btitle = QLabel(self.brightness)
        self.btitle.setObjectName(u"btitle")

        self.gridLayout.addWidget(self.btitle, 0, 0, 1, 1)

        self.bvalue = QLabel(self.brightness)
        self.bvalue.setObjectName(u"bvalue")
        self.bvalue.setLayoutDirection(Qt.LeftToRight)
        self.bvalue.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.bvalue, 0, 1, 1, 1)


        self.verticalLayout.addWidget(self.brightness)

        self.contrast = QWidget(Form)
        self.contrast.setObjectName(u"contrast")
        self.gridLayout_2 = QGridLayout(self.contrast)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.cslider = QSlider(self.contrast)
        self.cslider.setObjectName(u"cslider")
        self.cslider.setMaximum(100)
        self.cslider.setOrientation(Qt.Horizontal)

        self.gridLayout_2.addWidget(self.cslider, 2, 0, 1, 2)

        self.ctitle = QLabel(self.contrast)
        self.ctitle.setObjectName(u"ctitle")

        self.gridLayout_2.addWidget(self.ctitle, 0, 0, 1, 1)

        self.cvalue = QLabel(self.contrast)
        self.cvalue.setObjectName(u"cvalue")
        self.cvalue.setLayoutDirection(Qt.LeftToRight)
        self.cvalue.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_2.addWidget(self.cvalue, 0, 1, 1, 1)


        self.verticalLayout.addWidget(self.contrast)

        self.sharpness = QWidget(Form)
        self.sharpness.setObjectName(u"sharpness")
        self.gridLayout_3 = QGridLayout(self.sharpness)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.sslider = QSlider(self.sharpness)
        self.sslider.setObjectName(u"sslider")
        self.sslider.setMaximum(10)
        self.sslider.setPageStep(2)
        self.sslider.setOrientation(Qt.Horizontal)

        self.gridLayout_3.addWidget(self.sslider, 2, 0, 1, 2)

        self.stitle = QLabel(self.sharpness)
        self.stitle.setObjectName(u"stitle")

        self.gridLayout_3.addWidget(self.stitle, 0, 0, 1, 1)

        self.svalue = QLabel(self.sharpness)
        self.svalue.setObjectName(u"svalue")
        self.svalue.setLayoutDirection(Qt.LeftToRight)
        self.svalue.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_3.addWidget(self.svalue, 0, 1, 1, 1)


        self.verticalLayout.addWidget(self.sharpness)

        self.bluelight = QWidget(Form)
        self.bluelight.setObjectName(u"bluelight")
        self.gridLayout_4 = QGridLayout(self.bluelight)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.blslider = QSlider(self.bluelight)
        self.blslider.setObjectName(u"blslider")
        self.blslider.setMaximum(10)
        self.blslider.setPageStep(2)
        self.blslider.setOrientation(Qt.Horizontal)

        self.gridLayout_4.addWidget(self.blslider, 2, 0, 1, 2)

        self.bltitle = QLabel(self.bluelight)
        self.bltitle.setObjectName(u"bltitle")

        self.gridLayout_4.addWidget(self.bltitle, 0, 0, 1, 1)

        self.blvalue = QLabel(self.bluelight)
        self.blvalue.setObjectName(u"blvalue")
        self.blvalue.setLayoutDirection(Qt.LeftToRight)
        self.blvalue.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout_4.addWidget(self.blvalue, 0, 1, 1, 1)


        self.verticalLayout.addWidget(self.bluelight)

        self.colortemp = QWidget(Form)
        self.colortemp.setObjectName(u"colortemp")
        self.verticalLayout_2 = QVBoxLayout(self.colortemp)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.cttitle = QLabel(self.colortemp)
        self.cttitle.setObjectName(u"cttitle")

        self.verticalLayout_2.addWidget(self.cttitle)

        self.toolButton = QToolButton(self.colortemp)
        self.toolButton.setObjectName(u"toolButton")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolButton.sizePolicy().hasHeightForWidth())
        self.toolButton.setSizePolicy(sizePolicy)
        self.toolButton.setPopupMode(QToolButton.MenuButtonPopup)
        self.toolButton.setToolButtonStyle(Qt.ToolButtonIconOnly)
        self.toolButton.setAutoRaise(False)
        self.toolButton.setArrowType(Qt.NoArrow)

        self.verticalLayout_2.addWidget(self.toolButton)


        self.verticalLayout.addWidget(self.colortemp)


        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Monitor Control", None))
        self.btitle.setText(QCoreApplication.translate("Form", u"Brightness", None))
        self.bvalue.setText(QCoreApplication.translate("Form", u"0", None))
        self.ctitle.setText(QCoreApplication.translate("Form", u"Contrast", None))
        self.cvalue.setText(QCoreApplication.translate("Form", u"0", None))
        self.stitle.setText(QCoreApplication.translate("Form", u"Sharpness", None))
        self.svalue.setText(QCoreApplication.translate("Form", u"0", None))
        self.bltitle.setText(QCoreApplication.translate("Form", u"BlueLight", None))
        self.blvalue.setText(QCoreApplication.translate("Form", u"0", None))
        self.cttitle.setText(QCoreApplication.translate("Form", u"COLOR TEMPERATURE", None))
        self.toolButton.setText(QCoreApplication.translate("Form", u"PROFILE", None))
    # retranslateUi




import sys
from qtpy import QtWidgets

from .monctl import MyWindow

if __name__ == "__main__":
    app = QtWidgets.QApplication([])

    widget = MyWindow()
    widget.show()

    sys.exit(app.exec_())


import usb.core
import usb.util
from time import sleep

class G27Q:
	_dev=None	# Device Handler
	_usb_delay = 50/1000 # 50 ms sleep after every usb op

	_VID=0x2109	# (VIA Labs, Inc.)
	_PID=0x8883	# USB Billboard Device
	properties={
		"Brightness": {
			"min"	: 0,
			"max"	: 100,
			"code"	: [0x10, 0x00]
		},
		"Contrast": {
			"min"	: 0,
			"max"	: 100,
			"code"	: [0x12, 0x00]
		},
		"Sharpness": {
			"min"	: 0,
			"max"	: 10,
			"code"	: [0x87, 0x00]
		},
		"BlueLight": {
			"min"	: 0,
			"max"	: 10,
			"code"	: [0xe0, 0x0b]
		},
		"ColorMode": {
			"min"	: 0,
			"max"	: 3,
			"code"	: [0xe0, 0x03]
		},
		"Red": {
			"min"	: 0,
			"max"	: 100,
			"code"	: [0xe0, 0x04]
		},
		"Green": {
			"min"	: 0,
			"max"	: 100,
			"code"	: [0xe0, 0x05]
		},
		"Blue": {
			"min"	: 0,
			"max"	: 100,
			"code"	: [0xe0, 0x06]
		},
	}

	def __enter__(self):
		# Find device
		self._dev=usb.core.find(idVendor=self._VID, idProduct=self._PID)
		if self._dev is None:
			raise IOError(f"Device VID_{self._VID}&PID_{self._PID} not found")
		
		# Detach kernel driver
		self._had_driver = False
		try:
			if self._dev.is_kernel_driver_active(0):
				self._dev.detach_kernel_driver(0)
				self._had_driver = True
		except Exception as e:
			pass
		
		# Set config (1 as discovered with Wireshark)
		self._dev.set_configuration(1)
		return self

	# Optionally reattach kernel driver
	def __exit__(self, exc_type, exc_val, exc_tb):
		# Reattach kernel driver
		if self._had_driver:
			self._dev.attach_kernel_driver(0)
		# Release device
		usb.util.dispose_resources(self._dev)

	def usb_write(self, b_request: int, w_value: int, w_index: int, message: bytes):
		bm_request_type = 0x40
		if not self._dev.ctrl_transfer(bm_request_type, b_request, w_value, w_index, message) == len(message):
			raise IOError("Transferred message length mismatch")
		sleep(self._usb_delay)

	def usb_read(self, b_request: int, w_value: int, w_index: int, msg_length: int):
		bm_request_type = 0xC0
		data = self._dev.ctrl_transfer(bm_request_type, b_request, w_value, w_index, msg_length)
		sleep(self._usb_delay)
		return data

	def get_property(self, property: str):
		if not self.properties[property]:
			raise IOError("No property with that name")

		self.usb_write(
			b_request=178,
			w_value=0,
			w_index=0,
			message=bytearray([0x6E, 0x51, 0x83, 0x01])
				+ bytearray(self.properties[property]['code']),
		)
		data = self.usb_read(b_request=162, w_value=0, w_index=111, msg_length=12)
		return data[10]
	
	def set_property(self, property: str, value: int):
		if not self.properties[property]:
			raise IOError("No property with that name")

		self.usb_write(
			b_request=178,
			w_value=0,
			w_index=0,
			message=bytearray([0x6E, 0x51, 0x84, 0x03]
				+ self.properties[property]['code']
				+ [max(self.properties[property]['min'], min(self.properties[property]['max'], value))]),
		)